@@ -29,7 +29,7 @@ distribution or http://www.openidenabled.com/
 """)
     sys.exit(1)
 
-from openid import sreg
+from openid.extensions import sreg
 from openid.server import server
 from openid.store.filestore import FileOpenIDStore
 from openid.consumer import discover
@@ -170,7 +170,7 @@ class ServerHandler(BaseHTTPRequestHandler):
         if self.user is None:
             return False
 
-        if identity_url != self.server.base_url + self.user:
+        if identity_url != self.server.base_url + 'id/' + self.user:
             return False
 
         key = (identity_url, trust_root)
@@ -194,8 +194,8 @@ class ServerHandler(BaseHTTPRequestHandler):
             response = self.server.openid.handleRequest(request)
             self.displayResponse(response)
 
-    def addSRegResponse(self, response):
-        sreg_req = sreg.SRegRequest.fromOpenIDRequest(request.message)
+    def addSRegResponse(self, request, response):
+        sreg_req = sreg.SRegRequest.fromOpenIDRequest(request)
 
         # In a real application, this data would be user-specific,
         # and the user should be asked for permission to release
@@ -209,7 +209,7 @@ class ServerHandler(BaseHTTPRequestHandler):
 
     def approved(self, request, identifier=None):
         response = request.answer(True, identity=identifier)
-        self.addSRegResponse(response)
+        self.addSRegResponse(request, response)
         return response
 
     def handleCheckIDRequest(self, request):
@@ -335,6 +335,9 @@ class ServerHandler(BaseHTTPRequestHandler):
 
     def showDecidePage(self, request):
         id_url_base = self.server.base_url+'id/'
+        # XXX: This may break if there are any synonyms for id_url_base,
+        # such as referring to it by IP address or a CNAME.
+        assert request.identity.startswith(id_url_base), repr((request.identity, id_url_base))
         expected_user = request.identity[len(id_url_base):]
 
         if request.idSelect(): # We are being asked to select an ID
@@ -551,7 +554,7 @@ class ServerHandler(BaseHTTPRequestHandler):
         if self.user is None:
             user_link = '<a href="/login">not logged in</a>.'
         else:
-            user_link = 'logged in as <a href="/%s">%s</a>.<br /><a href="/loginsubmit?submit=true&success_to=/login">Log out</a>' % \
+            user_link = 'logged in as <a href="/id/%s">%s</a>.<br /><a href="/loginsubmit?submit=true&success_to=/login">Log out</a>' % \
                         (self.user, self.user)
 
         body = ''
