@@ -190,14 +190,14 @@ USING THIS LIBRARY
 
 import cgi
 import copy
-from urlparse import urlparse
+from urlparse import urlparse, urldefrag
 
 from openid import fetchers
 
 from openid.consumer.discover import discover, OpenIDServiceEndpoint, \
      DiscoveryFailure, OPENID_1_0_TYPE, OPENID_1_1_TYPE, OPENID_2_0_TYPE
 from openid.message import Message, OPENID_NS, OPENID2_NS, OPENID1_NS, \
-     IDENTIFIER_SELECT, no_default
+     IDENTIFIER_SELECT, no_default, BARE_NS
 from openid import cryptutil
 from openid import oidutil
 from openid.association import Association, default_negotiator, \
@@ -212,6 +212,7 @@ __all__ = ['AuthRequest', 'Consumer', 'SuccessResponse',
            'SUCCESS', 'FAILURE', 'CANCEL', 'SETUP_NEEDED',
            ]
 
+
 def makeKVPost(request_message, server_url):
     """Make a Direct Request to an OpenID Provider and return the
     result as a Message object.
@@ -224,18 +225,37 @@ def makeKVPost(request_message, server_url):
     # XXX: TESTME
     resp = fetchers.fetch(server_url, body=request_message.toURLEncoded())
 
-    response_message = Message.fromKVForm(resp.body)
-    if resp.status == 400:
+    # Process response in separate function that can be shared by async code.
+    return _httpResponseToMessage(resp, server_url)
+
+
+def _httpResponseToMessage(response, server_url):
+    """Adapt a POST response to a Message.
+
+    @type response: L{openid.fetchers.HTTPResponse}
+    @param response: Result of a POST to an OpenID endpoint.
+
+    @rtype: L{openid.message.Message}
+
+    @raises openid.fetchers.HTTPFetchingError: if the server returned a
+        status of other than 200 or 400.
+
+    @raises ServerError: if the server returned an OpenID error.
+    """
+    # Should this function be named Message.fromHTTPResponse instead?
+    response_message = Message.fromKVForm(response.body)
+    if response.status == 400:
         raise ServerError.fromMessage(response_message)
 
-    elif resp.status != 200:
+    elif response.status != 200:
         fmt = 'bad status code from server %s: %s'
-        error_message = fmt % (server_url, resp.status)
+        error_message = fmt % (server_url, response.status)
         raise fetchers.HTTPFetchingError(error_message)
 
     return response_message
 
 
+
 class Consumer(object):
     """An OpenID consumer implementation that performs discovery and
     does session management.
@@ -363,7 +383,7 @@ class Consumer(object):
 
         return auth_req
 
-    def complete(self, query, return_to=None):
+    def complete(self, query, return_to):
         """Called to interpret the server's response to an OpenID
         request. It is called in step 4 of the flow described in the
         consumer overview.
@@ -389,12 +409,14 @@ class Consumer(object):
         """
 
         endpoint = self.session.get(self._token_key)
-        if endpoint is None:
-            response = FailureResponse(None, 'No session state found')
-        else:
-            message = Message.fromPostArgs(query)
-            response = self.consumer.complete(message, endpoint, return_to)
+
+        message = Message.fromPostArgs(query)
+        response = self.consumer.complete(message, endpoint, return_to)
+
+        try:
             del self.session[self._token_key]
+        except KeyError:
+            pass
 
         if (response.status in ['success', 'cancel'] and
             response.identity_url is not None):
@@ -404,7 +426,7 @@ class Consumer(object):
                               self.session_key_prefix)
             # This is OK to do even if we did not do discovery in
             # the first place.
-            disco.cleanup()
+            disco.cleanup(force=True)
 
         return response
 
@@ -495,6 +517,19 @@ class TypeURIMismatch(ProtocolError):
     """A protocol error arising from type URIs mismatching
     """
 
+    def __init__(self, expected, endpoint):
+        ProtocolError.__init__(self, expected, endpoint)
+        self.expected = expected
+        self.endpoint = endpoint
+
+    def __str__(self):
+        s = '<%s.%s: Required type %s not found in %s for endpoint %s>' % (
+            self.__class__.__module__, self.__class__.__name__,
+            self.expected, self.endpoint.type_uris, self.endpoint)
+        return s
+
+
+
 class ServerError(Exception):
     """Exception that is raised when the server returns a 400 response
     code to a direct request."""
@@ -536,6 +571,11 @@ class GenericConsumer(object):
     # OpenID query parameter names.
     openid1_nonce_query_arg_name = 'janrain_nonce'
 
+    # Another query parameter that gets added to the return_to for
+    # OpenID 1; if the user's session state is lost, use this claimed
+    # identifier to do discovery when verifying the response.
+    openid1_return_to_identifier_name = 'openid1_claimed_id'
+
     session_types = {
         'DH-SHA1':DiffieHellmanSHA1ConsumerSession,
         'DH-SHA256':DiffieHellmanSHA256ConsumerSession,
@@ -559,45 +599,57 @@ class GenericConsumer(object):
 
         request = AuthRequest(service_endpoint, assoc)
         request.return_to_args[self.openid1_nonce_query_arg_name] = mkNonce()
+
+        if request.message.isOpenID1():
+            request.return_to_args[self.openid1_return_to_identifier_name] = \
+                request.endpoint.claimed_id
+
         return request
 
-    def complete(self, message, endpoint, return_to=None):
+    def complete(self, message, endpoint, return_to):
         """Process the OpenID message, using the specified endpoint
         and return_to URL as context. This method will handle any
         OpenID message that is sent to the return_to URL.
         """
         mode = message.getArg(OPENID_NS, 'mode', '<No mode set>')
 
-        if return_to is not None:
-            if not self._checkReturnTo(message, return_to):
-                return FailureResponse(endpoint,
-                                       "openid.return_to does not match return URL")
+        modeMethod = getattr(self, '_complete_' + mode,
+                             self._completeInvalid)
 
-        if mode == 'cancel':
-            return CancelResponse(endpoint)
-        elif mode == 'error':
-            error = message.getArg(OPENID_NS, 'error')
-            contact = message.getArg(OPENID_NS, 'contact')
-            reference = message.getArg(OPENID_NS, 'reference')
+        return modeMethod(message, endpoint, return_to)
 
-            return FailureResponse(endpoint, error, contact=contact,
-                                   reference=reference)
-        elif message.isOpenID2() and mode == 'setup_needed':
-            return SetupNeededResponse(endpoint)
+    def _complete_cancel(self, message, endpoint, _):
+        return CancelResponse(endpoint)
 
-        elif mode == 'id_res':
-            try:
-                self._checkSetupNeeded(message)
-            except SetupNeededError, why:
-                return SetupNeededResponse(endpoint, why.user_setup_url)
-            else:
-                try:
-                    return self._doIdRes(message, endpoint)
-                except (ProtocolError, DiscoveryFailure), why:
-                    return FailureResponse(endpoint, why[0])
+    def _complete_error(self, message, endpoint, _):
+        error = message.getArg(OPENID_NS, 'error')
+        contact = message.getArg(OPENID_NS, 'contact')
+        reference = message.getArg(OPENID_NS, 'reference')
+
+        return FailureResponse(endpoint, error, contact=contact,
+                               reference=reference)
+
+    def _complete_setup_needed(self, message, endpoint, _):
+        if not message.isOpenID2():
+            return self._completeInvalid(message, endpoint, _)
+
+        return SetupNeededResponse(endpoint)
+
+    def _complete_id_res(self, message, endpoint, return_to):
+        try:
+            self._checkSetupNeeded(message)
+        except SetupNeededError, why:
+            return SetupNeededResponse(endpoint, why.user_setup_url)
         else:
-            return FailureResponse(endpoint,
-                                   'Invalid openid.mode: %r' % (mode,))
+            try:
+                return self._doIdRes(message, endpoint, return_to)
+            except (ProtocolError, DiscoveryFailure), why:
+                return FailureResponse(endpoint, why[0])
+
+    def _completeInvalid(self, message, endpoint, _):
+        mode = message.getArg(OPENID_NS, 'mode', '<No mode set>')
+        return FailureResponse(endpoint,
+                               'Invalid openid.mode: %r' % (mode,))
 
     def _checkReturnTo(self, message, return_to):
         """Check an OpenID message and its openid.return_to value
@@ -644,7 +696,7 @@ class GenericConsumer(object):
             if user_setup_url is not None:
                 raise SetupNeededError(user_setup_url)
 
-    def _doIdRes(self, message, endpoint):
+    def _doIdRes(self, message, endpoint, return_to):
         """Handle id_res responses that are not cancellations of
         immediate mode requests.
 
@@ -663,47 +715,43 @@ class GenericConsumer(object):
 
         @returntype: L{Response}
         """
-        signed_list_str = message.getArg(OPENID_NS, 'signed')
-        if signed_list_str is None:
-            raise ProtocolError("Response missing signed list")
-
-        signed_list = signed_list_str.split(',')
-
         # Checks for presence of appropriate fields (and checks
         # signed list fields)
-        self._idResCheckForFields(message, signed_list)
+        self._idResCheckForFields(message)
+
+        if not self._checkReturnTo(message, return_to):
+            raise ProtocolError(
+                "return_to does not match return URL. Expected %r, got %r"
+                % (return_to, message.getArg(OPENID_NS, 'return_to')))
+
 
         # Verify discovery information:
         endpoint = self._verifyDiscoveryResults(message, endpoint)
+        oidutil.log("Received id_res response from %s using association %s" %
+                    (endpoint.server_url,
+                     message.getArg(OPENID_NS, 'assoc_handle')))
 
         self._idResCheckSignature(message, endpoint.server_url)
 
-        response_identity = message.getArg(OPENID_NS, 'identity')
-
         # Will raise a ProtocolError if the nonce is bad
         self._idResCheckNonce(message, endpoint)
 
+        signed_list_str = message.getArg(OPENID_NS, 'signed', no_default)
+        signed_list = signed_list_str.split(',')
         signed_fields = ["openid." + s for s in signed_list]
         return SuccessResponse(endpoint, message, signed_fields)
 
     def _idResGetNonceOpenID1(self, message, endpoint):
-        """Extract the nonce from an OpenID 1 response
+        """Extract the nonce from an OpenID 1 response.  Return the
+        nonce from the BARE_NS since we independently check the
+        return_to arguments are the same as those in the response
+        message.
 
         See the openid1_nonce_query_arg_name class variable
 
         @returns: The nonce as a string or None
         """
-        return_to = message.getArg(OPENID1_NS, 'return_to', None)
-        if return_to is None:
-            return None
-
-        parsed_url = urlparse(return_to)
-        query = parsed_url[4]
-        for k, v in cgi.parse_qsl(query):
-            if k == self.openid1_nonce_query_arg_name:
-                return v
-
-        return None
+        return message.getArg(BARE_NS, self.openid1_nonce_query_arg_name)
 
     def _idResCheckNonce(self, message, endpoint):
         if message.isOpenID1():
@@ -754,14 +802,14 @@ class GenericConsumer(object):
             if not self._checkAuth(message, server_url):
                 raise ProtocolError('Server denied check_authentication')
 
-    def _idResCheckForFields(self, message, signed_list):
+    def _idResCheckForFields(self, message):
         # XXX: this should be handled by the code that processes the
         # response (that is, if a field is missing, we should not have
         # to explicitly check that it's present, just make sure that
         # the fields are actually being used by the rest of the code
         # in tests). Although, which fields are signed does need to be
         # checked somewhere.
-        basic_fields = ['return_to', 'assoc_handle', 'sig']
+        basic_fields = ['return_to', 'assoc_handle', 'sig', 'signed']
         basic_sig_fields = ['return_to', 'identity']
 
         require_fields = {
@@ -780,6 +828,9 @@ class GenericConsumer(object):
             if not message.hasKey(OPENID_NS, field):
                 raise ProtocolError('Missing required field %r' % (field,))
 
+        signed_list_str = message.getArg(OPENID_NS, 'signed', no_default)
+        signed_list = signed_list_str.split(',')
+
         for field in require_sigs[message.getOpenIDNamespace()]:
             # Field is present and not in signed list
             if message.hasKey(OPENID_NS, field) and field not in signed_list:
@@ -793,12 +844,14 @@ class GenericConsumer(object):
         message = Message.fromPostArgs(query)
         return_to = message.getArg(OPENID_NS, 'return_to')
 
-        # XXX: this should be checked by _idResCheckForFields
-        if not return_to:
-            raise ProtocolError("no openid.return_to in query %r" % (query,))
+        if return_to is None:
+            raise ProtocolError('Response has no return_to')
+
         parsed_url = urlparse(return_to)
         rt_query = parsed_url[4]
-        for rt_key, rt_value in cgi.parse_qsl(rt_query):
+        parsed_args = cgi.parse_qsl(rt_query)
+
+        for rt_key, rt_value in parsed_args:
             try:
                 value = query[rt_key]
                 if rt_value != value:
@@ -809,6 +862,13 @@ class GenericConsumer(object):
                 format = "return_to parameter %s absent from query %r"
                 raise ProtocolError(format % (rt_key, query))
 
+        # Make sure all non-OpenID arguments in the response are also
+        # in the signed return_to.
+        bare_args = message.getArgs(BARE_NS)
+        for pair in bare_args.iteritems():
+            if pair not in parsed_args:
+                raise ProtocolError("Parameter %s not in return_to URL" % (pair[0],))
+
     _verifyReturnToArgs = staticmethod(_verifyReturnToArgs)
 
     def _verifyDiscoveryResults(self, resp_msg, endpoint=None):
@@ -818,6 +878,8 @@ class GenericConsumer(object):
 
         @param endpoint: The endpoint that resulted from doing discovery
         @param resp_msg: The id_res message object
+
+        @returns: the verified endpoint
         """
         if resp_msg.getOpenIDNamespace() == OPENID2_NS:
             return self._verifyDiscoveryResultsOpenID2(resp_msg, endpoint)
@@ -857,36 +919,45 @@ class GenericConsumer(object):
         # again. This covers not using sessions, OP identifier
         # endpoints and responses that didn't match the original
         # request.
-        elif not endpoint:
+        if not endpoint:
             oidutil.log('No pre-discovered information supplied.')
-            return self._discoverAndVerify(to_match)
-
-        elif to_match.claimed_id != endpoint.claimed_id:
-            oidutil.log('Mismatched pre-discovered session data. '
-                        'Claimed ID in session=%s, in assertion=%s' %
-                        (endpoint.claimed_id, to_match.claimed_id))
-            return self._discoverAndVerify(to_match)
-
-        # The claimed ID matches, so we use the endpoint that we
-        # discovered in initiation. This should be the most common
-        # case.
+            endpoint = self._discoverAndVerify(to_match)
         else:
-            self._verifyDiscoverySingle(endpoint, to_match)
-            return endpoint
+            # The claimed ID matches, so we use the endpoint that we
+            # discovered in initiation. This should be the most common
+            # case.
+            try:
+                self._verifyDiscoverySingle(endpoint, to_match)
+            except ProtocolError, e:
+                oidutil.log("Error attempting to use stored discovery information: " +
+                            str(e))
+                oidutil.log("Attempting discovery to verify endpoint")
+                endpoint = self._discoverAndVerify(to_match)
+
+        # The endpoint we return should have the claimed ID from the
+        # message we just verified, fragment and all.
+        if endpoint.claimed_id != to_match.claimed_id:
+            endpoint = copy.copy(endpoint)
+            endpoint.claimed_id = to_match.claimed_id
+        return endpoint
 
     def _verifyDiscoveryResultsOpenID1(self, resp_msg, endpoint):
-        if endpoint is None:
+        claimed_id = resp_msg.getArg(BARE_NS, self.openid1_return_to_identifier_name)
+
+        if endpoint is None and claimed_id is None:
             raise RuntimeError(
                 'When using OpenID 1, the claimed ID must be supplied, '
                 'either by passing it through as a return_to parameter '
                 'or by using a session, and supplied to the GenericConsumer '
                 'as the argument to complete()')
+        elif endpoint is not None and claimed_id is None:
+            claimed_id = endpoint.claimed_id
 
         to_match = OpenIDServiceEndpoint()
         to_match.type_uris = [OPENID_1_1_TYPE]
         to_match.local_id = resp_msg.getArg(OPENID1_NS, 'identity')
         # Restore delegate information from the initiation phase
-        to_match.claimed_id = endpoint.claimed_id
+        to_match.claimed_id = claimed_id
 
         if to_match.local_id is None:
             raise ProtocolError('Missing required field openid.identity')
@@ -894,12 +965,24 @@ class GenericConsumer(object):
         to_match_1_0 = copy.copy(to_match)
         to_match_1_0.type_uris = [OPENID_1_0_TYPE]
 
-        try:
-            self._verifyDiscoverySingle(endpoint, to_match)
-        except TypeURIMismatch:
-            self._verifyDiscoverySingle(endpoint, to_match_1_0)
+        if endpoint is not None:
+            try:
+                try:
+                    self._verifyDiscoverySingle(endpoint, to_match)
+                except TypeURIMismatch:
+                    self._verifyDiscoverySingle(endpoint, to_match_1_0)
+            except ProtocolError, e:
+                oidutil.log("Error attempting to use stored discovery information: " +
+                            str(e))
+                oidutil.log("Attempting discovery to verify endpoint")
+            else:
+                return endpoint
 
-        return endpoint
+        # Endpoint is either bad (failed verification) or None
+        try:
+            return self._discoverAndVerify(to_match)
+        except TypeURIMismatch:
+            return self._discoverAndVerify(to_match_1_0)
 
     def _verifyDiscoverySingle(self, endpoint, to_match):
         """Verify that the given endpoint matches the information
@@ -918,14 +1001,16 @@ class GenericConsumer(object):
         # present in the discovered endpoint.
         for type_uri in to_match.type_uris:
             if not endpoint.usesExtension(type_uri):
-                raise TypeURIMismatch(
-                    'Required type %r not present' % (type_uri,))
+                raise TypeURIMismatch(type_uri, endpoint)
 
-        if to_match.claimed_id != endpoint.claimed_id:
+        # Fragments do not influence discovery, so we can't compare a
+        # claimed identifier with a fragment to discovered information.
+        defragged_claimed_id, _ = urldefrag(to_match.claimed_id)
+        if defragged_claimed_id != endpoint.claimed_id:
             raise ProtocolError(
                 'Claimed ID does not match (different subjects!), '
                 'Expected %s, got %s' %
-                (to_match.claimed_id, endpoint.claimed_id))
+                (defragged_claimed_id, endpoint.claimed_id))
 
         if to_match.getLocalID() != endpoint.getLocalID():
             raise ProtocolError('local_id mismatch. Expected %s, got %s' %
@@ -960,13 +1045,18 @@ class GenericConsumer(object):
         @returns: The result of performing discovery on the claimed
             identifier in `to_match'
 
-        @raises ProtocolError: when discovery fails.
+        @raises DiscoveryFailure: when discovery fails.
         """
         oidutil.log('Performing discovery on %s' % (to_match.claimed_id,))
         _, services = self._discover(to_match.claimed_id)
         if not services:
             raise DiscoveryFailure('No OpenID information found at %s' %
                                    (to_match.claimed_id,), None)
+        return self._verifyDiscoveredServices(services, to_match)
+
+
+    def _verifyDiscoveredServices(self, services, to_match):
+        """See @L{_discoverAndVerify}"""
 
         # Search the services resulting from discovery to find one
         # that matches the information from the assertion
@@ -975,7 +1065,7 @@ class GenericConsumer(object):
             try:
                 self._verifyDiscoverySingle(endpoint, to_match)
             except ProtocolError, why:
-                failure_messages.append(why[0])
+                failure_messages.append(str(why))
             else:
                 # It matches, so discover verification has
                 # succeeded. Return this endpoint.
@@ -991,6 +1081,11 @@ class GenericConsumer(object):
                 % (to_match.claimed_id,), None)
 
     def _checkAuth(self, message, server_url):
+        """Make a check_authentication request to verify this message.
+
+        @returns: True if the request is valid.
+        @rtype: bool
+        """
         oidutil.log('Using OpenID check_authentication')
         request = self._createCheckAuthRequest(message)
         if request is None:
@@ -1020,10 +1115,6 @@ class GenericConsumer(object):
         signed = message.getArg(OPENID_NS, 'signed')
         if signed:
             for k in signed.split(','):
-                if k == 'ns':
-                    check_args['ns'] = message.getOpenIDNamespace()
-                    continue
-
                 val = message.getAliasedArg(k)
 
                 # Signed value is missing
@@ -1085,10 +1176,7 @@ class GenericConsumer(object):
 
         @returns: a new association object
 
-        @rtype: openid.association.Association
-
-        @raises Exception: errors that the fetcher might raise. These are
-            intended to be propagated up to the library's entrance point.
+        @rtype: L{openid.association.Association}
         """
         # Get our preferred session/association type from the negotiatior.
         assoc_type, session_type = self.negotiator.getAllowedType()
@@ -1097,37 +1185,11 @@ class GenericConsumer(object):
             assoc = self._requestAssociation(
                 endpoint, assoc_type, session_type)
         except ServerError, why:
-            # Any error message whose code is not 'unsupported-type'
-            # should be considered a total failure.
-            if why.error_code != 'unsupported-type' or \
-                   why.message.isOpenID1():
-                oidutil.log(
-                    'Server error when requesting an association from %r: %s'
-                    % (endpoint.server_url, why.error_text))
-                return None
-
-            # The server didn't like the association/session type
-            # that we sent, and it sent us back a message that
-            # might tell us how to handle it.
-            oidutil.log(
-                'Unsupported association type %s: %s' % (assoc_type,
-                                                         why.error_text,))
-
-            # Extract the session_type and assoc_type from the
-            # error message
-            assoc_type = why.message.getArg(OPENID_NS, 'assoc_type')
-            session_type = why.message.getArg(OPENID_NS, 'session_type')
-
-            if assoc_type is None or session_type is None:
-                oidutil.log('Server responded with unsupported association '
-                            'session but did not supply a fallback.')
-                return None
-            elif not self.negotiator.isAllowed(assoc_type, session_type):
-                fmt = ('Server sent unsupported session/association type: '
-                       'session_type=%s, assoc_type=%s')
-                oidutil.log(fmt % (session_type, assoc_type))
-                return None
-            else:
+            supportedTypes = self._extractSupportedAssociationType(why,
+                                                                   endpoint,
+                                                                   assoc_type)
+            if supportedTypes is not None:
+                assoc_type, session_type = supportedTypes
                 # Attempt to create an association from the assoc_type
                 # and session_type that the server told us it
                 # supported.
@@ -1147,6 +1209,49 @@ class GenericConsumer(object):
         else:
             return assoc
 
+    def _extractSupportedAssociationType(self, server_error, endpoint,
+                                         assoc_type):
+        """Handle ServerErrors resulting from association requests.
+
+        @returns: If server replied with an C{unsupported-type} error,
+            return a tuple of supported C{association_type}, C{session_type}.
+            Otherwise logs the error and returns None.
+        @rtype: tuple or None
+        """
+        # Any error message whose code is not 'unsupported-type'
+        # should be considered a total failure.
+        if server_error.error_code != 'unsupported-type' or \
+               server_error.message.isOpenID1():
+            oidutil.log(
+                'Server error when requesting an association from %r: %s'
+                % (endpoint.server_url, server_error.error_text))
+            return None
+
+        # The server didn't like the association/session type
+        # that we sent, and it sent us back a message that
+        # might tell us how to handle it.
+        oidutil.log(
+            'Unsupported association type %s: %s' % (assoc_type,
+                                                     server_error.error_text,))
+
+        # Extract the session_type and assoc_type from the
+        # error message
+        assoc_type = server_error.message.getArg(OPENID_NS, 'assoc_type')
+        session_type = server_error.message.getArg(OPENID_NS, 'session_type')
+
+        if assoc_type is None or session_type is None:
+            oidutil.log('Server responded with unsupported association '
+                        'session but did not supply a fallback.')
+            return None
+        elif not self.negotiator.isAllowed(assoc_type, session_type):
+            fmt = ('Server sent unsupported session/association type: '
+                   'session_type=%s, assoc_type=%s')
+            oidutil.log(fmt % (session_type, assoc_type))
+            return None
+        else:
+            return assoc_type, session_type
+
+
     def _requestAssociation(self, endpoint, assoc_type, session_type):
         """Make and process one association request to this endpoint's
         OP endpoint URL.
@@ -1497,6 +1602,12 @@ class AuthRequest(object):
 
         if self.assoc:
             message.setArg(OPENID_NS, 'assoc_handle', self.assoc.handle)
+            assoc_log_msg = 'with assocication %s' % (self.assoc.handle,)
+        else:
+            assoc_log_msg = 'using stateless mode.'
+
+        oidutil.log("Generated %s request to %s %s" %
+                    (mode, self.endpoint.server_url, assoc_log_msg))
 
         return message
 
@@ -1675,6 +1786,22 @@ class SuccessResponse(Response):
         """
         return self.getSigned(OPENID_NS, 'return_to')
 
+    def __eq__(self, other):
+        return (
+            (self.endpoint == other.endpoint) and
+            (self.identity_url == other.identity_url) and
+            (self.message == other.message) and
+            (self.signed_fields == other.signed_fields) and
+            (self.status == other.status))
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def __repr__(self):
+        return '<%s.%s id=%r signed=%r>' % (
+            self.__class__.__module__,
+            self.__class__.__name__,
+            self.identity_url, self.signed_fields)
 
 
 class FailureResponse(Response):
