@@ -2,6 +2,17 @@
 """Functions to discover OpenID endpoints from identifiers.
 """
 
+__all__ = [
+    'DiscoveryFailure',
+    'OPENID_1_0_NS',
+    'OPENID_1_0_TYPE',
+    'OPENID_1_1_TYPE',
+    'OPENID_2_0_TYPE',
+    'OPENID_IDP_2_0_TYPE',
+    'OpenIDServiceEndpoint',
+    'discover',
+    ]
+
 import urlparse
 
 from openid import oidutil, fetchers, urinorm
@@ -159,6 +170,42 @@ class OpenIDServiceEndpoint(object):
 
     fromHTML = classmethod(fromHTML)
 
+
+    def fromXRDS(cls, uri, xrds):
+        """Parse the given document as XRDS looking for OpenID services.
+
+        @rtype: [OpenIDServiceEndpoint]
+
+        @raises XRDSError: When the XRDS does not parse.
+
+        @since: 2.0.2
+        """
+        return extractServices(uri, xrds, cls)
+
+    fromXRDS = classmethod(fromXRDS)
+
+
+    def fromDiscoveryResult(cls, discoveryResult):
+        """Create endpoints from a DiscoveryResult.
+
+        @type discoveryResult: L{DiscoveryResult}
+
+        @rtype: list of L{OpenIDServiceEndpoint}
+
+        @raises XRDSError: When the XRDS does not parse.
+
+        @since: 2.0.2
+        """
+        if discoveryResult.isXRDS():
+            method = cls.fromXRDS
+        else:
+            method = cls.fromHTML
+        return method(discoveryResult.normalized_uri,
+                      discoveryResult.response_text)
+
+    fromDiscoveryResult = classmethod(fromDiscoveryResult)
+
+
     def fromOPEndpointURL(cls, op_endpoint_url):
         """Construct an OP-Identifier OpenIDServiceEndpoint object for
         a given OP Endpoint URL
@@ -245,9 +292,11 @@ def normalizeURL(url):
     """Normalize a URL, converting normalization failures to
     DiscoveryFailure"""
     try:
-        return urinorm.urinorm(url)
+        normalized = urinorm.urinorm(url)
     except ValueError, why:
         raise DiscoveryFailure('Normalizing identifier: %s' % (why[0],), None)
+    else:
+        return urlparse.urldefrag(normalized)[0]
 
 def arrangeByType(service_list, preferred_types):
     """Rearrange service_list in a new list so services are ordered by
@@ -323,10 +372,9 @@ def discoverYadis(uri):
     response = yadisDiscover(uri)
 
     yadis_url = response.normalized_uri
+    body = response.response_text
     try:
-        openid_services = extractServices(
-            response.normalized_uri, response.response_text,
-            OpenIDServiceEndpoint)
+        openid_services = OpenIDServiceEndpoint.fromXRDS(yadis_url, body)
     except XRDSError:
         # Does not parse as a Yadis XRDS file
         openid_services = []
@@ -339,10 +387,8 @@ def discoverYadis(uri):
             # header, re-fetch the document without following the Yadis
             # header, with no Accept header.
             return discoverNoYadis(uri)
-        else:
-            body = response.response_text
 
-        # Try to parse the response as HTML to get OpenID 1.0/1.1
+        # Try to parse the response as HTML.
         # <link rel="...">
         openid_services = OpenIDServiceEndpoint.fromHTML(yadis_url, body)
 
