@@ -41,6 +41,10 @@ OPENID_NS = oidutil.Symbol('OpenID namespace')
 # with "openid."
 BARE_NS = oidutil.Symbol('Bare namespace')
 
+# Limit, in bytes, of identity provider and return_to URLs, including
+# response payload.  See OpenID 1.1 specification, Appendix D.
+OPENID1_URL_LIMIT = 2047
+
 # All OpenID protocol fields.  Used to check namespace aliases.
 OPENID_PROTOCOL_FIELDS = [
     'ns', 'mode', 'error', 'return_to', 'contact', 'reference',
@@ -444,6 +448,8 @@ class Message(object):
 
     def setArg(self, namespace, key, value):
         """Set a single argument in this namespace"""
+        assert key is not None
+        assert value is not None
         namespace = self._fixNS(namespace)
         self.args[(namespace, key)] = value
         if not (namespace is BARE_NS):
@@ -453,7 +459,7 @@ class Message(object):
         namespace = self._fixNS(namespace)
         del self.args[(namespace, key)]
 
-    def __str__(self):
+    def __repr__(self):
         return "<%s.%s %r>" % (self.__class__.__module__,
                                self.__class__.__name__,
                                self.args)
@@ -467,6 +473,16 @@ class Message(object):
 
 
     def getAliasedArg(self, aliased_key, default=None):
+        if aliased_key == 'ns':
+            return self.getOpenIDNamespace()
+
+        if aliased_key.startswith('ns.'):
+            uri = self.namespaces.getNamespaceURI(aliased_key[3:])
+            if uri is None:
+                return default
+            else:
+                return uri
+
         try:
             alias, key = aliased_key.split('.', 1)
         except ValueError:
