@@ -14,18 +14,44 @@ import urlparse
 from urllib import urlencode
 
 elementtree_modules = [
+    'lxml.etree',
     'xml.etree.cElementTree',
     'xml.etree.ElementTree',
     'cElementTree',
     'elementtree.ElementTree',
     ]
 
-def importElementTree():
-    for mod_name in elementtree_modules:
+def importElementTree(module_names=None):
+    """Find a working ElementTree implementation, trying the standard
+    places that such a thing might show up.
+
+    >>> ElementTree = importElementTree()
+
+    @param module_names: The names of modules to try to use as
+        ElementTree. Defaults to C{L{elementtree_modules}}
+
+    @returns: An ElementTree module
+    """
+    if module_names is None:
+        module_names = elementtree_modules
+
+    for mod_name in module_names:
         try:
-            return __import__(mod_name, None, None, ['unused'])
+            ElementTree = __import__(mod_name, None, None, ['unused'])
         except ImportError:
             pass
+        else:
+            # Make sure it can actually parse XML
+            try:
+                ElementTree.XML('<unused/>')
+            except (SystemExit, MemoryError, AssertionError):
+                raise
+            except:
+                why = sys.exc_info()[1]
+                log('Not using ElementTree library %r because it failed to '
+                    'parse a trivial document: %s' % (mod_name, why))
+            else:
+                return ElementTree
     else:
         raise
 
