@@ -28,9 +28,9 @@ Overview
     she does consent to releasing that information to the party making the
     request.
 
-    Examine the properties of the L{CheckIDRequest} object, and if
-    and when you've come to a decision, form a response by calling
-    L{CheckIDRequest.answer}.
+    Examine the properties of the L{CheckIDRequest} object, optionally
+    check L{CheckIDRequest.returnToVerified}, and and when you've come
+    to a decision, form a response by calling L{CheckIDRequest.answer}.
 
     Other types of requests relate to establishing associations between client
     and server and verifying the authenticity of previous communications.
@@ -45,7 +45,7 @@ OpenID Extensions
     in addition to authentication?  Version 2.0 of the OpenID
     protocol allows consumers to add extensions to their requests.
     For example, with sites using the U{Simple Registration
-    Extension<http://www.openidenabled.com/openid/simple-registration-extension/>},
+    Extension<http://openid.net/specs/openid-simple-registration-extension-1_0.html>},
     a user can agree to have their nickname and e-mail address sent to a
     site when they sign up.
 
@@ -64,6 +64,11 @@ OpenID Extensions
         # as well as a namespace declaration for the openid.sreg namespace
         response.fields.setArg('http://openid.net/sreg/1.0', 'timezone', 'America/Los_Angeles')
 
+    There are helper modules for a number of extensions, including
+    L{Attribute Exchange<openid.extensions.ax>},
+    L{PAPE<openid.extensions.pape>}, and
+    L{Simple Registration<openid.extensions.sreg>} in the L{openid.extensions}
+    package.
 
 Stores
 ======
@@ -81,11 +86,25 @@ Stores
 Upgrading
 =========
 
+From 1.0 to 1.1
+---------------
+
     The keys by which a server looks up associations in its store have changed
     in version 1.2 of this library.  If your store has entries created from
     version 1.0 code, you should empty it.
 
-    FIXME: add notes on 1.2 -> 2.0 upgrade here.
+From 1.1 to 2.0
+---------------
+
+    One of the additions to the OpenID protocol was a specified nonce
+    format for one-way nonces.  As a result, the nonce table in the store
+    has changed.  You'll need to run contrib/upgrade-store-1.1-to-2.0 to
+    upgrade your store, or you'll encounter errors about the wrong number
+    of columns in the oid_nonces table.
+
+    If you've written your own custom store or code that interacts
+    directly with it, you'll need to review the change notes in
+    L{openid.store.interface}.
 
 @group Requests: OpenIDRequest, AssociateRequest, CheckIDRequest,
     CheckAuthRequest
@@ -94,7 +113,7 @@ Upgrading
 
 @group HTTP Codes: HTTP_OK, HTTP_REDIRECT, HTTP_ERROR
 
-@group Response Encodings: ENCODE_KVFORM, ENCODE_URL
+@group Response Encodings: ENCODE_KVFORM, ENCODE_HTML_FORM, ENCODE_URL
 """
 
 import time, warnings
@@ -104,10 +123,10 @@ from openid import cryptutil
 from openid import oidutil
 from openid.dh import DiffieHellman
 from openid.store.nonce import mkNonce
-from openid.server.trustroot import TrustRoot
+from openid.server.trustroot import TrustRoot, verifyReturnTo
 from openid.association import Association, default_negotiator, getSecretSize
 from openid.message import Message, OPENID_NS, OPENID1_NS, \
-     OPENID2_NS, IDENTIFIER_SELECT
+     OPENID2_NS, IDENTIFIER_SELECT, OPENID1_URL_LIMIT
 
 HTTP_OK = 200
 HTTP_REDIRECT = 302
@@ -117,6 +136,7 @@ BROWSER_REQUEST_MODES = ['checkid_setup', 'checkid_immediate']
 
 ENCODE_KVFORM = ('kvform',)
 ENCODE_URL = ('URL/redirect',)
+ENCODE_HTML_FORM = ('HTML form',)
 
 UNUSED = None
 
@@ -658,6 +678,32 @@ class CheckIDRequest(OpenIDRequest):
         else:
             return True
 
+    def returnToVerified(self):
+        """Does the relying party publish the return_to URL for this
+        response under the realm? It is up to the provider to set a
+        policy for what kinds of realms should be allowed. This
+        return_to URL verification reduces vulnerability to data-theft
+        attacks based on open proxies, corss-site-scripting, or open
+        redirectors.
+
+        This check should only be performed after making sure that the
+        return_to URL matches the realm.
+
+        @see: L{trustRootValid}
+
+        @raises openid.yadis.discover.DiscoveryFailure: if the realm
+            URL does not support Yadis discovery (and so does not
+            support the verification process).
+
+        @returntype: bool
+
+        @returns: True if the realm publishes a document with the
+            return_to URL listed
+
+        @since: 2.0.2
+        """
+        return verifyReturnTo(self.trust_root, self.return_to)
+
     def answer(self, allow, server_url=None, identity=None, claimed_id=None):
         """Respond to this request.
 
@@ -768,11 +814,13 @@ class CheckIDRequest(OpenIDRequest):
 
             response.fields.updateArgs(OPENID_NS, {
                 'mode': mode,
-                'op_endpoint': server_url,
                 'return_to': self.return_to,
                 'response_nonce': mkNonce(),
                 })
 
+            if server_url:
+                response.fields.setArg(OPENID_NS, 'op_endpoint', server_url)
+
             if response_identity is not None:
                 response.fields.setArg(
                     OPENID_NS, 'identity', response_identity)
@@ -870,7 +918,7 @@ class OpenIDResponse(object):
     @ivar fields: My parameters as a dictionary with each key mapping to
         one value.  Keys are parameter names with no leading "C{openid.}".
         e.g.  "C{identity}" and "C{mac_key}", never "C{openid.identity}".
-    @type fields: dict
+    @type fields: L{openid.message.Message}
 
     @ivar signed: The names of the fields which should be signed.
     @type signed: list of str
@@ -898,6 +946,28 @@ class OpenIDResponse(object):
             self.fields)
 
 
+    def toFormMarkup(self):
+        """Returns the form markup for this response.
+
+        @returntype: str
+
+        @since: 2.0.2
+        """
+        return self.fields.toFormMarkup(
+            self.fields.getArg(OPENID_NS, 'return_to'))
+
+
+    def renderAsForm(self):
+        """Returns True if this response's encoding is
+        ENCODE_HTML_FORM.  Convenience method for server authors.
+
+        @returntype: bool
+
+        @since: 2.0.2
+        """
+        return self.whichEncoding() == ENCODE_HTML_FORM
+
+
     def needsSigning(self):
         """Does this response require signing?
 
@@ -911,10 +981,16 @@ class OpenIDResponse(object):
     def whichEncoding(self):
         """How should I be encoded?
 
-        @returns: one of ENCODE_URL or ENCODE_KVFORM.
+        @returns: one of ENCODE_URL, ENCODE_HTML_FORM, or ENCODE_KVFORM.
+
+        @change: 2.0.2 added the ENCODE_HTML_FORM response.
         """
         if self.request.mode in BROWSER_REQUEST_MODES:
-            return ENCODE_URL
+            if self.fields.getOpenIDNamespace() == OPENID2_NS and \
+               len(self.encodeToURL()) > OPENID1_URL_LIMIT:
+                return ENCODE_HTML_FORM
+            else:
+                return ENCODE_URL
         else:
             return ENCODE_KVFORM
 
@@ -1202,6 +1278,9 @@ class Encoder(object):
             location = response.encodeToURL()
             wr = self.responseFactory(code=HTTP_REDIRECT,
                                       headers={'location': location})
+        elif encode_as == ENCODE_HTML_FORM:
+            wr = self.responseFactory(code=HTTP_OK,
+                                      body=response.toFormMarkup())
         else:
             # Can't encode this to a protocol message.  You should probably
             # render it to HTML and show it to the user.
@@ -1545,6 +1624,13 @@ class ProtocolError(Exception):
     def encodeToKVForm(self):
         return self.toMessage().toKVForm()
 
+    def toFormMarkup(self):
+        """Encode to HTML form markup for POST.
+
+        @since: 2.0.2
+        """
+        return self.toMessage().toFormMarkup(self.getReturnTo())
+
     def whichEncoding(self):
         """How should I be encoded?
 
@@ -1553,7 +1639,11 @@ class ProtocolError(Exception):
             displayed to the user.
         """
         if self.hasReturnTo():
-            return ENCODE_URL
+            if self.openid_message.getOpenIDNamespace() == OPENID2_NS and \
+               len(self.encodeToURL()) > OPENID1_URL_LIMIT:
+                return ENCODE_HTML_FORM
+            else:
+                return ENCODE_URL
 
         if self.openid_message is None:
             return None
