@@ -1,10 +1,25 @@
+# -*- test-case-name: openid.test.test_rpverify -*-
 """
 This module contains the C{L{TrustRoot}} class, which helps handle
 trust root checking.  This module is used by the
 C{L{openid.server.server}} module, but it is also available to server
 implementers who wish to use it for additional trust root checking.
+
+It also implements relying party return_to URL verification, based on
+the realm.
 """
 
+__all__ = [
+    'TrustRoot',
+    'RP_RETURN_TO_URL_TYPE',
+    'extractReturnToURLs',
+    'returnToMatches',
+    'verifyReturnTo',
+    ]
+
+from openid import oidutil
+from openid import urinorm
+from openid.yadis import services
 from urlparse import urlparse, urlunparse
 
 ############################################
@@ -25,7 +40,27 @@ _top_level_domains = (
     ).split('|')
 
 
+class RealmVerificationRedirected(Exception):
+    """Attempting to verify this realm resulted in a redirect.
+
+    @since: 2.0.2
+    """
+    def __init__(self, relying_party_url, rp_url_after_redirects):
+        self.relying_party_url = relying_party_url
+        self.rp_url_after_redirects = rp_url_after_redirects
+
+    def __str__(self):
+        return ("Attempting to verify %r resulted in "
+                "redirect to %r" %
+                (self.relying_party_url,
+                 self.rp_url_after_redirects))
+
+
 def _parseURL(url):
+    try:
+        url = urinorm.urinorm(url)
+    except ValueError:
+        pass
     proto, netloc, path, params, query, frag = urlparse(url)
     if not path:
         # Python <2.4 does not parse URLs with no path properly
@@ -255,6 +290,27 @@ class TrustRoot(object):
 
     checkURL = classmethod(checkURL)
 
+    def buildDiscoveryURL(self):
+        """Return a discovery URL for this realm.
+
+        This function does not check to make sure that the realm is
+        valid. Its behaviour on invalid inputs is undefined.
+
+        @rtype: str
+
+        @returns: The URL upon which relying party discovery should be run
+            in order to verify the return_to URL
+
+        @since: 2.0.2
+        """
+        if self.wildcard:
+            # Use "www." in place of the star
+            assert self.host.startswith('.'), self.host
+            www_domain = 'www' + self.host
+            return '%s://%s%s' % (self.proto, www_domain, self.path)
+        else:
+            return self.unparsed
+
     def __repr__(self):
         return "TrustRoot('%s', '%s', '%s', '%s', '%s', '%s')" % (
             self.unparsed, self.proto, self.wildcard, self.host, self.port,
@@ -262,3 +318,106 @@ class TrustRoot(object):
 
     def __str__(self):
         return repr(self)
+
+# The URI for relying party discovery, used in realm verification.
+#
+# XXX: This should probably live somewhere else (like in
+# openid.consumer or openid.yadis somewhere)
+RP_RETURN_TO_URL_TYPE = 'http://specs.openid.net/auth/2.0/return_to'
+
+def _extractReturnURL(endpoint):
+    """If the endpoint is a relying party OpenID return_to endpoint,
+    return the endpoint URL. Otherwise, return None.
+
+    This function is intended to be used as a filter for the Yadis
+    filtering interface.
+
+    @see: C{L{openid.yadis.services}}
+    @see: C{L{openid.yadis.filters}}
+
+    @param endpoint: An XRDS BasicServiceEndpoint, as returned by
+        performing Yadis dicovery.
+
+    @returns: The endpoint URL or None if the endpoint is not a
+        relying party endpoint.
+    @rtype: str or NoneType
+    """
+    if endpoint.matchTypes([RP_RETURN_TO_URL_TYPE]):
+        return endpoint.uri
+    else:
+        return None
+
+def returnToMatches(allowed_return_to_urls, return_to):
+    """Is the return_to URL under one of the supplied allowed
+    return_to URLs?
+
+    @since: 2.0.2
+    """
+
+    for allowed_return_to in allowed_return_to_urls:
+        # A return_to pattern works the same as a realm, except that
+        # it's not allowed to use a wildcard. We'll model this by
+        # parsing it as a realm, and not trying to match it if it has
+        # a wildcard.
+
+        return_realm = TrustRoot.parse(allowed_return_to)
+        if (# Parses as a trust root
+            return_realm is not None and
+
+            # Does not have a wildcard
+            not return_realm.wildcard and
+
+            # Matches the return_to that we passed in with it
+            return_realm.validateURL(return_to)
+            ):
+            return True
+
+    # No URL in the list matched
+    return False
+
+def getAllowedReturnURLs(relying_party_url):
+    """Given a relying party discovery URL return a list of return_to URLs.
+
+    @since: 2.0.2
+    """
+    (rp_url_after_redirects, return_to_urls) = services.getServiceEndpoints(
+        relying_party_url, _extractReturnURL)
+
+    if rp_url_after_redirects != relying_party_url:
+        # Verification caused a redirect
+        raise RealmVerificationRedirected(
+            relying_party_url, rp_url_after_redirects)
+
+    return return_to_urls
+
+# _vrfy parameter is there to make testing easier
+def verifyReturnTo(realm_str, return_to, _vrfy=getAllowedReturnURLs):
+    """Verify that a return_to URL is valid for the given realm.
+
+    This function builds a discovery URL, performs Yadis discovery on
+    it, makes sure that the URL does not redirect, parses out the
+    return_to URLs, and finally checks to see if the current return_to
+    URL matches the return_to.
+
+    @raises DiscoveryFailure: When Yadis discovery fails
+    @returns: True if the return_to URL is valid for the realm
+
+    @since: 2.0.2
+    """
+    realm = TrustRoot.parse(realm_str)
+    if realm is None:
+        # The realm does not parse as a URL pattern
+        return False
+
+    try:
+        allowable_urls = _vrfy(realm.buildDiscoveryURL())
+    except RealmVerificationRedirected, err:
+        oidutil.log(str(err))
+        return False
+
+    if returnToMatches(allowable_urls, return_to):
+        return True
+    else:
+        oidutil.log("Failed to validate return_to %r for realm %r, was not "
+                    "in %s" % (return_to, realm_str, allowable_urls))
+        return False
