@@ -2,5 +2,5 @@
 This package contains the modules related to this library's use of
 persistent storage.
 
-@sort: interface, filestore, sqlstore, dumbstore
+@sort: interface, filestore, sqlstore, memstore
 """
