@@ -14,7 +14,7 @@ from openid.consumer.consumer import \
      SuccessResponse, FailureResponse, SetupNeededResponse, CancelResponse, \
      DiffieHellmanSHA1ConsumerSession, Consumer, PlainTextConsumerSession, \
      SetupNeededError, DiffieHellmanSHA256ConsumerSession, ServerError, \
-     ProtocolError
+     ProtocolError, _httpResponseToMessage
 from openid import association
 from openid.server.server import \
      PlainTextServerSession, DiffieHellmanSHA1ServerSession
@@ -173,22 +173,20 @@ def _test_success(server_url, user_url, delegate_url, links, immediate=False):
         assert new_return_to.startswith(return_to)
         assert redirect_url.startswith(server_url)
 
-        nonce_key = consumer.openid1_nonce_query_arg_name
-        nonce = request.return_to_args[nonce_key]
-
-        query = {
-            nonce_key:nonce,
+        parsed = urlparse.urlparse(new_return_to)
+        query = parseQuery(parsed[4])
+        query.update({
             'openid.mode':'id_res',
             'openid.return_to':new_return_to,
             'openid.identity':delegate_url,
             'openid.assoc_handle':fetcher.assoc_handle,
-            }
+            })
 
         assoc = store.getAssociation(server_url, fetcher.assoc_handle)
 
         message = Message.fromPostArgs(query)
         message = assoc.signMessage(message)
-        info = consumer.complete(message, request.endpoint)
+        info = consumer.complete(message, request.endpoint, new_return_to)
         assert info.status == SUCCESS, info.message
         assert info.identity_url == user_url
 
@@ -215,12 +213,13 @@ http_server_url = 'http://server.example.com/'
 consumer_url = 'http://consumer.example.com/'
 https_server_url = 'https://server.example.com/'
 
-class TestSuccess(unittest.TestCase):
+class TestSuccess(unittest.TestCase, CatchLogs):
     server_url = http_server_url
     user_url = 'http://www.example.com/user.html'
     delegate_url = 'http://consumer.example.com/user'
 
     def setUp(self):
+        CatchLogs.setUp(self)
         self.links = '<link rel="openid.server" href="%s" />' % (
             self.server_url,)
 
@@ -228,6 +227,9 @@ class TestSuccess(unittest.TestCase):
                                '<link rel="openid.delegate" href="%s" />') % (
             self.server_url, self.delegate_url)
 
+    def tearDown(self):
+        CatchLogs.tearDown(self)
+
     def test_nodelegate(self):
         _test_success(self.server_url, self.user_url,
                       self.user_url, self.links)
@@ -261,10 +263,12 @@ class TestConstruct(unittest.TestCase):
         self.failUnlessRaises(TypeError, GenericConsumer)
 
 
-class TestIdRes(unittest.TestCase):
+class TestIdRes(unittest.TestCase, CatchLogs):
     consumer_class = GenericConsumer
 
     def setUp(self):
+        CatchLogs.setUp(self)
+
         self.store = memstore.MemoryStore()
         self.consumer = self.consumer_class(self.store)
         self.return_to = "nonny"
@@ -281,6 +285,14 @@ class TestIdRes(unittest.TestCase):
             return endpoint
         self.consumer._verifyDiscoveryResults = dummyVerifyDiscover
 
+    def disableReturnToChecking(self):
+        def checkReturnTo(unused1, unused2):
+            return True
+        self.consumer._checkReturnTo = checkReturnTo
+        complete = self.consumer.complete
+        def callCompleteWithoutReturnTo(message, endpoint):
+            return complete(message, endpoint, None)
+        self.consumer.complete = callCompleteWithoutReturnTo
 
 class TestIdResCheckSignature(TestIdRes):
     def setUp(self):
@@ -379,21 +391,29 @@ class TestComplete(TestIdRes):
 
         self.consumer._checkSetupNeeded = raiseSetupNeeded
 
-        response = self.consumer.complete(message, None)
+        response = self.consumer.complete(message, None, None)
         self.failUnlessEqual(SETUP_NEEDED, response.status)
         self.failUnless(setup_url_sentinel is response.setup_url)
 
     def test_cancel(self):
         message = Message.fromPostArgs({'openid.mode': 'cancel'})
+        self.disableReturnToChecking()
         r = self.consumer.complete(message, self.endpoint)
         self.failUnlessEqual(r.status, CANCEL)
         self.failUnless(r.identity_url == self.endpoint.claimed_id)
 
+    def test_cancel_with_return_to(self):
+        message = Message.fromPostArgs({'openid.mode': 'cancel'})
+        r = self.consumer.complete(message, self.endpoint, self.return_to)
+        self.failUnlessEqual(r.status, CANCEL)
+        self.failUnless(r.identity_url == self.endpoint.claimed_id)
+
     def test_error(self):
         msg = 'an error message'
         message = Message.fromPostArgs({'openid.mode': 'error',
                  'openid.error': msg,
                  })
+        self.disableReturnToChecking()
         r = self.consumer.complete(message, self.endpoint)
         self.failUnlessEqual(r.status, FAILURE)
         self.failUnless(r.identity_url == self.endpoint.claimed_id)
@@ -406,6 +426,7 @@ class TestComplete(TestIdRes):
                  'openid.error': msg,
                  'openid.contact': contact,
                  })
+        self.disableReturnToChecking()
         r = self.consumer.complete(message, self.endpoint)
         self.failUnlessEqual(r.status, FAILURE)
         self.failUnless(r.identity_url == self.endpoint.claimed_id)
@@ -421,7 +442,7 @@ class TestComplete(TestIdRes):
                  'openid.error': msg, 'openid.reference': reference,
                  'openid.contact': contact, 'openid.ns': OPENID2_NS,
                  })
-        r = self.consumer.complete(message, self.endpoint)
+        r = self.consumer.complete(message, self.endpoint, None)
         self.failUnlessEqual(r.status, FAILURE)
         self.failUnless(r.identity_url == self.endpoint.claimed_id)
         self.failUnless(r.contact == contact)
@@ -430,7 +451,7 @@ class TestComplete(TestIdRes):
 
     def test_noMode(self):
         message = Message.fromPostArgs({})
-        r = self.consumer.complete(message, self.endpoint)
+        r = self.consumer.complete(message, self.endpoint, None)
         self.failUnlessEqual(r.status, FAILURE)
         self.failUnless(r.identity_url == self.endpoint.claimed_id)
 
@@ -440,9 +461,17 @@ class TestComplete(TestIdRes):
         # *check_auth* failed, not because it's missing an arg, exactly.
         message = Message.fromPostArgs({'openid.mode': 'id_res'})
         self.failUnlessRaises(ProtocolError, self.consumer._doIdRes,
-                              message, self.endpoint)
+                              message, self.endpoint, None)
 
     def test_idResURLMismatch(self):
+        class VerifiedError(Exception): pass
+
+        def discoverAndVerify(_to_match):
+            raise VerifiedError
+
+        self.consumer._discoverAndVerify = discoverAndVerify
+        self.disableReturnToChecking()
+
         message = Message.fromPostArgs(
             {'openid.mode': 'id_res',
              'openid.return_to': 'return_to (just anything)',
@@ -452,13 +481,13 @@ class TestComplete(TestIdRes):
              'openid.signed': 'identity,return_to',
              })
         self.consumer.store = GoodAssocStore()
-        r = self.consumer.complete(message, self.endpoint)
-        self.failUnlessEqual(r.status, FAILURE)
-        self.failUnlessEqual(r.identity_url, self.consumer_id)
-        self.failUnless(r.message.startswith('local_id mismatch'),
-                        r.message)
 
+        self.failUnlessRaises(VerifiedError,
+                              self.consumer.complete,
+                              message, self.endpoint)
 
+        self.failUnlessLogMatches('Error attempting to use stored',
+                                  'Attempting discovery')
 
 class TestCompleteMissingSig(unittest.TestCase, CatchLogs):
 
@@ -486,6 +515,7 @@ class TestCompleteMissingSig(unittest.TestCase, CatchLogs):
         self.endpoint = OpenIDServiceEndpoint()
         self.endpoint.server_url = self.server_url
         self.endpoint.claimed_id = claimed_id
+        self.consumer._checkReturnTo = lambda unused1, unused2 : True
 
     def tearDown(self):
         CatchLogs.tearDown(self)
@@ -496,7 +526,7 @@ class TestCompleteMissingSig(unittest.TestCase, CatchLogs):
             return endpoint
 
         self.consumer._verifyDiscoveryResults = _vrfy
-        r = self.consumer.complete(self.message, self.endpoint)
+        r = self.consumer.complete(self.message, self.endpoint, None)
         self.failUnlessSuccess(r)
 
 
@@ -505,31 +535,31 @@ class TestCompleteMissingSig(unittest.TestCase, CatchLogs):
         self.message.delArg(OPENID_NS, 'claimed_id')
         self.endpoint.claimed_id = None
         self.message.setArg(OPENID_NS, 'signed', 'return_to,response_nonce,assoc_handle')
-        r = self.consumer.complete(self.message, self.endpoint)
+        r = self.consumer.complete(self.message, self.endpoint, None)
         self.failUnlessSuccess(r)
 
 
     def test_idResMissingIdentitySig(self):
         self.message.setArg(OPENID_NS, 'signed', 'return_to,response_nonce,assoc_handle,claimed_id')
-        r = self.consumer.complete(self.message, self.endpoint)
+        r = self.consumer.complete(self.message, self.endpoint, None)
         self.failUnlessEqual(r.status, FAILURE)
 
 
     def test_idResMissingReturnToSig(self):
         self.message.setArg(OPENID_NS, 'signed', 'identity,response_nonce,assoc_handle,claimed_id')
-        r = self.consumer.complete(self.message, self.endpoint)
+        r = self.consumer.complete(self.message, self.endpoint, None)
         self.failUnlessEqual(r.status, FAILURE)
 
 
     def test_idResMissingAssocHandleSig(self):
         self.message.setArg(OPENID_NS, 'signed', 'identity,response_nonce,return_to,claimed_id')
-        r = self.consumer.complete(self.message, self.endpoint)
+        r = self.consumer.complete(self.message, self.endpoint, None)
         self.failUnlessEqual(r.status, FAILURE)
 
 
     def test_idResMissingClaimedIDSig(self):
         self.message.setArg(OPENID_NS, 'signed', 'identity,response_nonce,return_to,assoc_handle')
-        r = self.consumer.complete(self.message, self.endpoint)
+        r = self.consumer.complete(self.message, self.endpoint, None)
         self.failUnlessEqual(r.status, FAILURE)
 
 
@@ -717,7 +747,8 @@ class IdResCheckForFieldsTest(TestIdRes):
     def mkSuccessTest(openid_args, signed_list):
         def test(self):
             message = Message.fromOpenIDArgs(openid_args)
-            self.consumer._idResCheckForFields(message, signed_list)
+            message.setArg(OPENID_NS, 'signed', ','.join(signed_list))
+            self.consumer._idResCheckForFields(message)
         return test
 
     test_openid1Success = mkSuccessTest(
@@ -751,16 +782,13 @@ class IdResCheckForFieldsTest(TestIdRes):
         ['return_to', 'response_nonce', 'identity',
          'claimed_id', 'assoc_handle'])
 
-    def mkFailureTest(openid_args, signed_list, sig_fail=False):
+    def mkFailureTest(openid_args, signed_list):
         def test(self):
             message = Message.fromOpenIDArgs(openid_args)
             try:
-                self.consumer._idResCheckForFields(message, signed_list)
+                self.consumer._idResCheckForFields(message)
             except ProtocolError, why:
-                if sig_fail:
-                    self.failUnless(why[0].endswith('not signed'))
-                else:
-                    self.failUnless(why[0].startswith('Missing required'))
+                self.failUnless(why[0].startswith('Missing required'))
             else:
                 self.fail('Expected an error, but none occurred')
         return test
@@ -771,8 +799,7 @@ class IdResCheckForFieldsTest(TestIdRes):
          'sig':'a signature',
          'identity':'someone',
          },
-        ['identity'],
-        sig_fail=True)
+        ['identity'])
 
     test_openid1Missing_identitySig = mkFailureTest(
         {'return_to':'return',
@@ -780,8 +807,7 @@ class IdResCheckForFieldsTest(TestIdRes):
          'sig':'a signature',
          'identity':'someone',
          },
-        ['return_to'],
-        sig_fail=True)
+        ['return_to'])
 
     test_openid1MissingReturnTo = mkFailureTest(
         {'assoc_handle':'assoc handle',
@@ -812,8 +838,10 @@ class CheckNonceVerifyTest(TestIdRes, CatchLogs):
 
     def test_openid1Success(self):
         """use consumer-generated nonce"""
-        self.return_to = 'http://rt.unittest/?nonce=%s' % (mkNonce(),)
+        nonce_value = mkNonce()
+        self.return_to = 'http://rt.unittest/?nonce=%s' % (nonce_value,)
         self.response = Message.fromOpenIDArgs({'return_to': self.return_to})
+        self.response.setArg(BARE_NS, 'nonce', nonce_value)
         self.consumer._idResCheckNonce(self.response, self.endpoint)
         self.failUnlessLogEmpty()
 
@@ -921,8 +949,9 @@ class TestCheckAuthTriggered(TestIdRes, CatchLogs):
             'openid.sig': GOODSIG,
             'openid.signed': 'identity,return_to',
             })
+        self.disableReturnToChecking()
         try:
-            result = self.consumer._doIdRes(message, self.endpoint)
+            result = self.consumer._doIdRes(message, self.endpoint, None)
         except CheckAuthHappened:
             pass
         else:
@@ -937,7 +966,7 @@ class TestCheckAuthTriggered(TestIdRes, CatchLogs):
         assoc = association.Association(
             'handle', 'secret', issued, lifetime, 'HMAC-SHA1')
         self.store.storeAssociation(self.server_url, assoc)
-
+        self.disableReturnToChecking()
         message = Message.fromPostArgs({
             'openid.return_to':self.return_to,
             'openid.identity':self.server_id,
@@ -946,7 +975,7 @@ class TestCheckAuthTriggered(TestIdRes, CatchLogs):
             'openid.signed': 'identity,return_to',
             })
         try:
-            result = self.consumer._doIdRes(message, self.endpoint)
+            result = self.consumer._doIdRes(message, self.endpoint, None)
         except CheckAuthHappened:
             pass
         else:
@@ -970,8 +999,9 @@ class TestCheckAuthTriggered(TestIdRes, CatchLogs):
             'openid.sig': GOODSIG,
             'openid.signed': 'identity,return_to',
             })
+        self.disableReturnToChecking()
         self.failUnlessRaises(ProtocolError, self.consumer._doIdRes,
-                              message, self.endpoint)
+                              message, self.endpoint, None)
 
     def test_newerAssoc(self):
         lifetime = 1000
@@ -996,7 +1026,8 @@ class TestCheckAuthTriggered(TestIdRes, CatchLogs):
 
         message = Message.fromOpenIDArgs(query)
         message = good_assoc.signMessage(message)
-        info = self.consumer._doIdRes(message, self.endpoint)
+        self.disableReturnToChecking()
+        info = self.consumer._doIdRes(message, self.endpoint, None)
         self.failUnlessEqual(info.status, SUCCESS, info.message)
         self.failUnlessEqual(self.consumer_id, info.identity_url)
 
@@ -1033,6 +1064,15 @@ class TestReturnToArgs(unittest.TestCase):
         # no return value, success is assumed if there are no exceptions.
         self.consumer._verifyReturnToArgs(query)
 
+    def test_returnToArgsUnexpectedArg(self):
+        query = {
+            'openid.mode': 'id_res',
+            'openid.return_to': 'http://example.com/',
+            'foo': 'bar',
+            }
+        # no return value, success is assumed if there are no exceptions.
+        self.failUnlessRaises(ProtocolError,
+                              self.consumer._verifyReturnToArgs, query)
 
     def test_returnToMismatch(self):
         query = {
@@ -1082,11 +1122,7 @@ class TestReturnToArgs(unittest.TestCase):
 
         for bad in bad_return_tos:
             m.setArg(OPENID_NS, 'return_to', bad)
-            result = self.consumer.complete(m, endpoint, return_to)
-            self.failUnless(isinstance(result, FailureResponse), \
-                            "Expected FailureResponse, got %r for %s" % (result, bad))
-            self.failUnless(result.message == \
-                            "openid.return_to does not match return URL")
+            self.failIf(self.consumer._checkReturnTo(m, return_to))
 
     def test_completeGoodReturnTo(self):
         """Test GenericConsumer.complete()'s handling of good
@@ -1177,10 +1213,13 @@ class TestCheckAuth(unittest.TestCase, CatchLogs):
     def test_signedList(self):
         query = Message.fromOpenIDArgs({
             'mode': 'id_res',
+            'ns': OPENID2_NS,
             'sig': 'rabbits',
             'identity': '=example',
             'assoc_handle': 'munchkins',
-            'signed': 'identity,mode',
+            'ns.sreg': 'urn:sreg',
+            'sreg.email': 'bogus@example.com',
+            'signed': 'identity,mode,ns.sreg,sreg.email',
             'foo': 'bar',
             })
         expected = Message.fromOpenIDArgs({
@@ -1188,7 +1227,9 @@ class TestCheckAuth(unittest.TestCase, CatchLogs):
             'sig': 'rabbits',
             'assoc_handle': 'munchkins',
             'identity': '=example',
-            'signed': 'identity,mode'
+            'signed': 'identity,mode,ns.sreg,sreg.email',
+            'ns.sreg': 'urn:sreg',
+            'sreg.email': 'bogus@example.com',
             })
         args = self.consumer._createCheckAuthRequest(query)
         self.failUnlessEqual(args.toPostArgs(), expected.toPostArgs())
@@ -1326,7 +1367,7 @@ class StubConsumer(object):
         self.endpoint = service
         return auth_req
 
-    def complete(self, message, endpoint, return_to=None):
+    def complete(self, message, endpoint, return_to):
         assert endpoint is self.endpoint
         return self.response
 
@@ -1352,8 +1393,8 @@ class ConsumerTest(unittest.TestCase):
                                    association.SessionNegotiator))
         self.failUnlessEqual([],
                              self.consumer.consumer.negotiator.allowed_types)
-        self.consumer.setAssociationPreference([('FOO', 'BAR')])
-        self.failUnlessEqual([('FOO', 'BAR')],
+        self.consumer.setAssociationPreference([('HMAC-SHA1', 'DH-SHA1')])
+        self.failUnlessEqual([('HMAC-SHA1', 'DH-SHA1')],
                              self.consumer.consumer.negotiator.allowed_types)
 
     def withDummyDiscovery(self, callable, dummy_getNextService):
@@ -1420,8 +1461,17 @@ class ConsumerTest(unittest.TestCase):
         self.failUnless(result.endpoint is self.endpoint)
 
     def test_completeEmptySession(self):
-        response = self.consumer.complete({})
+        text = "failed complete"
+
+        def checkEndpoint(message, endpoint, return_to):
+            self.failUnless(endpoint is None)
+            return FailureResponse(endpoint, text)
+
+        self.consumer.consumer.complete = checkEndpoint
+
+        response = self.consumer.complete({}, None)
         self.failUnlessEqual(response.status, FAILURE)
+        self.failUnlessEqual(response.message, text)
         self.failUnless(response.identity_url is None)
 
     def _doResp(self, auth_req, exp_resp):
@@ -1433,11 +1483,13 @@ class ConsumerTest(unittest.TestCase):
 
         # endpoint is stored in the session
         self.failUnless(self.session)
-        resp = self.consumer.complete({})
+        resp = self.consumer.complete({}, None)
 
         # All responses should have the same identity URL, and the
         # session should be cleaned out
-        self.failUnless(resp.identity_url is self.identity_url)
+        if self.endpoint.claimed_id != IDENTIFIER_SELECT:
+            self.failUnless(resp.identity_url is self.identity_url)
+
         self.failIf(self.consumer._token_key in self.session)
 
         # Expected status response
@@ -1506,6 +1558,26 @@ class ConsumerTest(unittest.TestCase):
             SetupNeededResponse(self.endpoint, setup_url))
         self.failUnless(resp.setup_url is setup_url)
 
+    def test_successDifferentURL(self):
+        """
+        Be sure that the session gets cleaned up when the response is
+        successful and has a different URL than the one in the
+        request.
+        """
+        # Set up a request endpoint describing an IDP URL
+        self.identity_url = 'http://idp.url/'
+        self.endpoint.claimed_id = self.endpoint.local_id = IDENTIFIER_SELECT
+
+        # Use a response endpoint with a different URL (asserted by
+        # the IDP)
+        resp_endpoint = OpenIDServiceEndpoint()
+        resp_endpoint.claimed_id = "http://user.url/"
+
+        resp = self._doRespDisco(
+            True,
+            mkSuccess(resp_endpoint, {}))
+        self.failUnless(self.discovery.getManager(force=True) is None)
+
     def test_begin(self):
         self.discovery.createManager([self.endpoint], self.identity_url)
         # Should not raise an exception
@@ -1552,7 +1624,8 @@ class IDPDrivenTest(unittest.TestCase):
             return discovered_endpoint
         self.consumer._verifyDiscoveryResults = verifyDiscoveryResults
         self.consumer._idResCheckNonce = lambda *args: True
-        response = self.consumer._doIdRes(message, self.endpoint)
+        self.consumer._checkReturnTo = lambda unused1, unused2 : True
+        response = self.consumer._doIdRes(message, self.endpoint, None)
 
         self.failUnlessSuccess(response)
         self.failUnlessEqual(response.identity_url, "=directed_identifier")
@@ -1573,8 +1646,9 @@ class IDPDrivenTest(unittest.TestCase):
         def verifyDiscoveryResults(identifier, endpoint):
             raise DiscoveryFailure("PHREAK!", None)
         self.consumer._verifyDiscoveryResults = verifyDiscoveryResults
+        self.consumer._checkReturnTo = lambda unused1, unused2 : True
         self.failUnlessRaises(DiscoveryFailure, self.consumer._doIdRes,
-                              message, self.endpoint)
+                              message, self.endpoint, None)
 
 
     def failUnlessSuccess(self, response):
@@ -1618,6 +1692,14 @@ class TestDiscoveryVerification(unittest.TestCase):
 
 
     def test_otherServer(self):
+        text = "verify failed"
+
+        def discoverAndVerify(to_match):
+            self.failUnlessEqual(self.identifier, to_match.claimed_id)
+            raise ProtocolError(text)
+
+        self.consumer._discoverAndVerify = discoverAndVerify
+
         # a set of things without the stuff
         endpoint = OpenIDServiceEndpoint()
         endpoint.type_uris = [OPENID_2_0_TYPE]
@@ -1629,25 +1711,33 @@ class TestDiscoveryVerification(unittest.TestCase):
             r = self.consumer._verifyDiscoveryResults(self.message, endpoint)
         except ProtocolError, e:
             # Should we make more ProtocolError subclasses?
-            self.failUnless('OP Endpoint mismatch' in str(e), e)
+            self.failUnless(str(e), text)
         else:
             self.fail("expected ProtocolError, %r returned." % (r,))
             
 
     def test_foreignDelegate(self):
+        text = "verify failed"
+
+        def discoverAndVerify(to_match):
+            self.failUnlessEqual(self.identifier, to_match.claimed_id)
+            raise ProtocolError(text)
+
+        self.consumer._discoverAndVerify = discoverAndVerify
+
         # a set of things with the server stuff but other delegate
         endpoint = OpenIDServiceEndpoint()
         endpoint.type_uris = [OPENID_2_0_TYPE]
         endpoint.claimed_id = self.identifier
         endpoint.server_url = self.server_url
         endpoint.local_id = "http://unittest/juan-carlos"
+
         try:
             r = self.consumer._verifyDiscoveryResults(self.message, endpoint)
         except ProtocolError, e:
-            self.failUnless('local_id mismatch' in str(e), e)
+            self.failUnlessEqual(str(e), text)
         else:
-            self.fail("expected ProtocolError, %r returned." % (r,))
-
+            self.fail("Exepected ProtocolError, %r returned" % (r,))
 
     def test_nothingDiscovered(self):
         # a set of no things.
@@ -1899,5 +1989,47 @@ class TestAddExtension(unittest.TestCase):
         ar.addExtension(ext)
         ext_args = ar.message.getArgs(ext.ns_uri)
         self.failUnlessEqual(ext.getExtensionArgs(), ext_args)
+
+
+
+class TestKVPost(unittest.TestCase):
+    def setUp(self):
+        self.server_url = 'http://unittest/%s' % (self.id(),)
+
+    def test_200(self):
+        from openid.fetchers import HTTPResponse
+        response = HTTPResponse()
+        response.status = 200
+        response.body = "foo:bar\nbaz:quux\n"
+        r = _httpResponseToMessage(response, self.server_url)
+        expected_msg = Message.fromOpenIDArgs({'foo':'bar','baz':'quux'})
+        self.failUnlessEqual(expected_msg, r)
+
+
+    def test_400(self):
+        response = HTTPResponse()
+        response.status = 400
+        response.body = "error:bonk\nerror_code:7\n"
+        try:
+            r = _httpResponseToMessage(response, self.server_url)
+        except ServerError, e:
+            self.failUnlessEqual(e.error_text, 'bonk')
+            self.failUnlessEqual(e.error_code, '7')
+        else:
+            self.fail("Expected ServerError, got return %r" % (r,))
+
+
+    def test_500(self):
+        # 500 as an example of any non-200, non-400 code.
+        response = HTTPResponse()
+        response.status = 500
+        response.body = "foo:bar\nbaz:quux\n"
+        self.failUnlessRaises(fetchers.HTTPFetchingError,
+                              _httpResponseToMessage, response,
+                              self.server_url)
+
+
+
+
 if __name__ == '__main__':
     unittest.main()
