@@ -3,7 +3,7 @@
 from openid.server import server
 from openid import association, cryptutil, oidutil
 from openid.message import Message, OPENID_NS, OPENID2_NS, OPENID1_NS, \
-     IDENTIFIER_SELECT, no_default
+     IDENTIFIER_SELECT, no_default, OPENID1_URL_LIMIT
 from openid.store import memstore
 import cgi
 
@@ -53,6 +53,71 @@ class TestProtocolError(unittest.TestCase):
         result_args = cgi.parse_qs(result_args)
         self.failUnlessEqual(result_args, expected_args)
 
+    def test_browserWithReturnTo_OpenID2_GET(self):
+        return_to = "http://rp.unittest/consumer"
+        # will be a ProtocolError raised by Decode or CheckIDRequest.answer
+        args = Message.fromPostArgs({
+            'openid.ns': OPENID2_NS,
+            'openid.mode': 'monkeydance',
+            'openid.identity': 'http://wagu.unittest/',
+            'openid.claimed_id': 'http://wagu.unittest/',
+            'openid.return_to': return_to,
+            })
+        e = server.ProtocolError(args, "plucky")
+        self.failUnless(e.hasReturnTo())
+        expected_args = {
+            'openid.ns': [OPENID2_NS],
+            'openid.mode': ['error'],
+            'openid.error': ['plucky'],
+            }
+
+        rt_base, result_args = e.encodeToURL().split('?', 1)
+        result_args = cgi.parse_qs(result_args)
+        self.failUnlessEqual(result_args, expected_args)
+
+    def test_browserWithReturnTo_OpenID2_POST(self):
+        return_to = "http://rp.unittest/consumer" + ('x' * OPENID1_URL_LIMIT)
+        # will be a ProtocolError raised by Decode or CheckIDRequest.answer
+        args = Message.fromPostArgs({
+            'openid.ns': OPENID2_NS,
+            'openid.mode': 'monkeydance',
+            'openid.identity': 'http://wagu.unittest/',
+            'openid.claimed_id': 'http://wagu.unittest/',
+            'openid.return_to': return_to,
+            })
+        e = server.ProtocolError(args, "plucky")
+        self.failUnless(e.hasReturnTo())
+        expected_args = {
+            'openid.ns': [OPENID2_NS],
+            'openid.mode': ['error'],
+            'openid.error': ['plucky'],
+            }
+
+        self.failUnless(e.whichEncoding() == server.ENCODE_HTML_FORM)
+        self.failUnless(e.toFormMarkup() == e.toMessage().toFormMarkup(
+            args.getArg(OPENID_NS, 'return_to')))
+
+    def test_browserWithReturnTo_OpenID1_exceeds_limit(self):
+        return_to = "http://rp.unittest/consumer" + ('x' * OPENID1_URL_LIMIT)
+        # will be a ProtocolError raised by Decode or CheckIDRequest.answer
+        args = Message.fromPostArgs({
+            'openid.mode': 'monkeydance',
+            'openid.identity': 'http://wagu.unittest/',
+            'openid.return_to': return_to,
+            })
+        e = server.ProtocolError(args, "plucky")
+        self.failUnless(e.hasReturnTo())
+        expected_args = {
+            'openid.mode': ['error'],
+            'openid.error': ['plucky'],
+            }
+
+        self.failUnless(e.whichEncoding() == server.ENCODE_URL)
+
+        rt_base, result_args = e.encodeToURL().split('?', 1)
+        result_args = cgi.parse_qs(result_args)
+        self.failUnlessEqual(result_args, expected_args)
+
     def test_noReturnTo(self):
         # will be a ProtocolError raised by Decode or CheckIDRequest.answer
         args = Message.fromPostArgs({
@@ -442,6 +507,88 @@ class TestEncode(unittest.TestCase):
         self.store = memstore.MemoryStore()
         self.server = server.Server(self.store, self.op_endpoint)
 
+    def test_id_res_OpenID2_GET(self):
+        """
+        Check that when an OpenID 2 response does not exceed the
+        OpenID 1 message size, a GET response (i.e., redirect) is
+        issued.
+        """
+        request = server.CheckIDRequest(
+            identity = 'http://bombom.unittest/',
+            trust_root = 'http://burr.unittest/',
+            return_to = 'http://burr.unittest/999',
+            immediate = False,
+            op_endpoint = self.server.op_endpoint,
+            )
+        response = server.OpenIDResponse(request)
+        response.fields = Message.fromOpenIDArgs({
+            'ns': OPENID2_NS,
+            'mode': 'id_res',
+            'identity': request.identity,
+            'claimed_id': request.identity,
+            'return_to': request.return_to,
+            })
+
+        self.failIf(response.renderAsForm())
+        self.failUnless(response.whichEncoding() == server.ENCODE_URL)
+        webresponse = self.encode(response)
+        self.failUnless(webresponse.headers.has_key('location'))
+
+    def test_id_res_OpenID2_POST(self):
+        """
+        Check that when an OpenID 2 response exceeds the OpenID 1
+        message size, a POST response (i.e., an HTML form) is
+        returned.
+        """
+        request = server.CheckIDRequest(
+            identity = 'http://bombom.unittest/',
+            trust_root = 'http://burr.unittest/',
+            return_to = 'http://burr.unittest/999',
+            immediate = False,
+            op_endpoint = self.server.op_endpoint,
+            )
+        response = server.OpenIDResponse(request)
+        response.fields = Message.fromOpenIDArgs({
+            'ns': OPENID2_NS,
+            'mode': 'id_res',
+            'identity': request.identity,
+            'claimed_id': request.identity,
+            'return_to': 'x' * OPENID1_URL_LIMIT,
+            })
+
+        self.failUnless(response.renderAsForm())
+        self.failUnless(len(response.encodeToURL()) > OPENID1_URL_LIMIT)
+        self.failUnless(response.whichEncoding() == server.ENCODE_HTML_FORM)
+        webresponse = self.encode(response)
+        self.failUnlessEqual(webresponse.body, response.toFormMarkup())
+
+    def test_id_res_OpenID1_exceeds_limit(self):
+        """
+        Check that when an OpenID 1 response exceeds the OpenID 1
+        message size, a GET response is issued.  Technically, this
+        shouldn't be permitted by the library, but this test is in
+        place to preserve the status quo for OpenID 1.
+        """
+        request = server.CheckIDRequest(
+            identity = 'http://bombom.unittest/',
+            trust_root = 'http://burr.unittest/',
+            return_to = 'http://burr.unittest/999',
+            immediate = False,
+            op_endpoint = self.server.op_endpoint,
+            )
+        response = server.OpenIDResponse(request)
+        response.fields = Message.fromOpenIDArgs({
+            'mode': 'id_res',
+            'identity': request.identity,
+            'return_to': 'x' * OPENID1_URL_LIMIT,
+            })
+
+        self.failIf(response.renderAsForm())
+        self.failUnless(len(response.encodeToURL()) > OPENID1_URL_LIMIT)
+        self.failUnless(response.whichEncoding() == server.ENCODE_URL)
+        webresponse = self.encode(response)
+        self.failUnlessEqual(webresponse.headers['location'], response.encodeToURL())
+
     def test_id_res(self):
         request = server.CheckIDRequest(
             identity = 'http://bombom.unittest/',
@@ -625,8 +772,6 @@ class TestSigningEncode(unittest.TestCase):
         self.response.fields.setArg(OPENID_NS, 'sig', 'priorSig==')
         self.failUnlessRaises(server.AlreadySigned, self.encode, self.response)
 
-
-
 class TestCheckID(unittest.TestCase):
     def setUp(self):
         self.op_endpoint = 'http://endpoint.unittest/'
@@ -661,6 +806,44 @@ class TestCheckID(unittest.TestCase):
 
         self.failUnless(request.trustRootValid())
 
+    def test_returnToVerified_callsVerify(self):
+        """Make sure that verifyReturnTo is calling the trustroot
+        function verifyReturnTo
+        """
+        def withVerifyReturnTo(new_verify, callable):
+            old_verify = server.verifyReturnTo
+            try:
+                server.verifyReturnTo = new_verify
+                return callable()
+            finally:
+                server.verifyReturnTo = old_verify
+
+        # Ensure that exceptions are passed through
+        sentinel = Exception()
+        def vrfyExc(trust_root, return_to):
+            self.failUnlessEqual(self.request.trust_root, trust_root)
+            self.failUnlessEqual(self.request.return_to, return_to)
+            raise sentinel
+
+        try:
+            withVerifyReturnTo(vrfyExc, self.request.returnToVerified)
+        except Exception, e:
+            self.failUnless(e is sentinel, e)
+
+        # Ensure that True and False are passed through unchanged
+        def constVerify(val):
+            def verify(trust_root, return_to):
+                self.failUnlessEqual(self.request.trust_root, trust_root)
+                self.failUnlessEqual(self.request.return_to, return_to)
+                return val
+            return verify
+
+        for val in [True, False]:
+            self.failUnlessEqual(
+                val,
+                withVerifyReturnTo(constVerify(val),
+                                   self.request.returnToVerified))
+
     def _expectAnswer(self, answer, identity=None, claimed_id=None):
         expected_list = [
             ('mode', 'id_res'),
@@ -683,8 +866,28 @@ class TestCheckID(unittest.TestCase):
 
         # One for nonce, one for ns
         self.failUnlessEqual(len(answer.fields.toPostArgs()),
-                             len(expected_list) + 2, answer.fields.toPostArgs())
-        
+                             len(expected_list) + 2,
+                             answer.fields.toPostArgs())
+
+    def _expectAnswerv1(self, answer, identity=None):
+        expected_list = [
+            ('mode', 'id_res'),
+            ('return_to', self.request.return_to),
+            ('identity', identity),
+            ]
+
+        for k, expected in expected_list:
+            actual = answer.fields.getArg(OPENID_NS, k)
+            self.failUnlessEqual(actual, expected, "%s: expected %s, got %s" % (k, expected, actual))
+
+        self.failUnless(answer.fields.hasKey(OPENID_NS, 'response_nonce'))
+        self.failUnlessEqual(answer.fields.getOpenIDNamespace(), OPENID1_NS)
+
+        # One for nonce
+        self.failUnlessEqual(len(answer.fields.toPostArgs()),
+                             len(expected_list) + 1,
+                             answer.fields.toPostArgs())
+
 
     def test_answerAllow(self):
         """Check the fields specified by "Positive Assertions"
@@ -804,6 +1007,19 @@ class TestCheckID(unittest.TestCase):
         self.failUnlessEqual(answer.request, self.request)
         self._expectAnswer(answer, self.request.identity)
 
+    def test_answerAllowNoEndpointOpenID1(self):
+        """Test .allow() with an OpenID 1.x Message on a CheckIDRequest
+        built without an op_endpoint parameter.
+        """
+        reqmessage = Message.fromOpenIDArgs({
+            'identity': 'http://bambam.unittest/',
+            'trust_root': 'http://bar.unittest/',
+            'return_to': 'http://bar.unittest/999',
+            })
+        self.request = server.CheckIDRequest.fromMessage(reqmessage, None)
+        answer = self.request.answer(True)
+        self._expectAnswerv1(answer, 'http://bambam.unittest/')
+
     def test_answerImmediateDenyOpenID2(self):
         """Look for mode=setup_needed in checkid_immediate negative
         response in OpenID 2 case.
